package com.yunzhi.marketing.xlk.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.marketing.base.BaseController;
import com.yunzhi.marketing.base.ResponseBean;
import com.yunzhi.marketing.common.CommConstant;
import com.yunzhi.marketing.common.StringUtils;
import com.yunzhi.marketing.entity.*;
import com.yunzhi.marketing.excel.IntentionUser;
import com.yunzhi.marketing.excel.handler.CustomCellWriteHandler;
import com.yunzhi.marketing.excel.searchHouse.AddedValue;
import com.yunzhi.marketing.excel.searchHouse.BuyRoom;
import com.yunzhi.marketing.excel.searchHouse.Overseas;
import com.yunzhi.marketing.excel.searchHouse.Rent;
import com.yunzhi.marketing.service.*;
import com.yunzhi.marketing.xlk.dto.SearchHouseDTO;
import com.yunzhi.marketing.xlk.dto.SearchHouseReplayDTO;
import com.yunzhi.marketing.xlk.entity.Curriculum;
import com.yunzhi.marketing.xlk.entity.SearchHouse;
import com.yunzhi.marketing.xlk.service.ISearchHouseService;
import com.yunzhi.marketing.xlk.vo.ExportSearchHouseVO;
import com.yunzhi.marketing.xlk.vo.SearchHouseVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 帮我找房  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2021-05-19
 */
@RestController
@RequestMapping("/api")
@Api(value = "帮我找房", tags = "xlk-帮我找房")
public class SearchHouseController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SearchHouseController.class);

    @Autowired
    public ISearchHouseService iSearchHouseService;

    @Autowired
    public ITaPersonService taPersonService;

    @Autowired
    private ITaBuildingService taBuildingService;

    @Autowired
    private ITdCityService tdCityService;

    @Autowired
    private ISysOrgParamsService iSysOrgParamsService;

    @Autowired
    private ITaBuildingImgService taBuildingImgService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "admin-获取帮我找房列表", notes = "admin-获取帮我找房列表")
    @RequestMapping(value="/admin/searchHouse",method= RequestMethod.GET)
    public ResponseBean searchHouseList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                        @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                        @RequestParam(value ="type") Integer type,
                                        @RequestParam(value ="name", required = false) String name,
                                        @RequestParam(value ="phone", required = false) String phone,
                                        @RequestParam(value ="status", required = false) String status,
                                        @RequestHeader("authorization") String token, HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        try {
            SearchHouseVO searchHouse = new SearchHouseVO();
            searchHouse.setOrgId(getOrgId(request));
            searchHouse.setType(type);
            searchHouse.setNickname(name);
            searchHouse.setPhone(phone);
            searchHouse.setStatus(status);
            //使用分页插件
            IPage<SearchHouseVO> pg = new Page<>(pageNum, pageSize);
            responseBean = iSearchHouseService.selectSearchHouseList(pg,searchHouse);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("searchHouseList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 导出数据列表
     * @return
     */
    @ApiOperation(value = "admin-导出帮我找房列表", notes = "admin-导出帮我找房列表")
    @RequestMapping(value="/admin/exportHouse",method= RequestMethod.GET)
    public void exportHouse(@RequestParam(value ="type") Integer type,
                                    @RequestParam(value ="name", required = false) String name,
                                    @RequestParam(value ="phone", required = false) String phone,
                                    @RequestParam(value ="status", required = false) String status,
                                    @RequestHeader("authorization") String token, HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.setContentType("application/octet-stream");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=帮我找房.xlsx");
        SearchHouseVO searchHouse = new SearchHouseVO();
        searchHouse.setOrgId(getOrgId(request));
        searchHouse.setType(type);
        searchHouse.setNickname(name);
        searchHouse.setPhone(phone);
        searchHouse.setStatus(status);
        List<ExportSearchHouseVO> result = iSearchHouseService.selectExportHouseList(searchHouse);

        if (searchHouse.getType() == 4) {
            ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream(), AddedValue.class).registerWriteHandler(new CustomCellWriteHandler()).build();
            // 设置 sheet, 同一个sheet只需要设置一次
            WriteSheet writeSheet = EasyExcel.writerSheet("帮我找房").build();
            List<AddedValue> list = new ArrayList<>();
            result.forEach(e -> {
                AddedValue addedValue = new AddedValue();
                BeanUtils.copyProperties(e,addedValue);
                list.add(addedValue);
            });
            excelWriter.write(list, writeSheet);
            // finish 会帮忙关闭流
            excelWriter.finish();
        }else if (searchHouse.getType() == 1) {
            ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream(), BuyRoom.class).registerWriteHandler(new CustomCellWriteHandler()).build();
            // 设置 sheet, 同一个sheet只需要设置一次
            WriteSheet writeSheet = EasyExcel.writerSheet("帮我找房").build();
            List<BuyRoom> list = new ArrayList<>();
            result.forEach(e -> {
                BuyRoom buyRoom = new BuyRoom();
                BeanUtils.copyProperties(e,buyRoom);
                list.add(buyRoom);
            });
            excelWriter.write(list, writeSheet);
            // finish 会帮忙关闭流
            excelWriter.finish();
        }else if (searchHouse.getType() == 3) {
            ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream(), Overseas.class).registerWriteHandler(new CustomCellWriteHandler()).build();
            // 设置 sheet, 同一个sheet只需要设置一次
            WriteSheet writeSheet = EasyExcel.writerSheet("帮我找房").build();
            List<Overseas> list = new ArrayList<>();
            result.forEach(e -> {
                Overseas overseas = new Overseas();
                BeanUtils.copyProperties(e,overseas);
                list.add(overseas);
            });
            excelWriter.write(list, writeSheet);
            // finish 会帮忙关闭流
            excelWriter.finish();
        }else if (searchHouse.getType() == 2) {
            ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream(), Rent.class).registerWriteHandler(new CustomCellWriteHandler()).build();
            // 设置 sheet, 同一个sheet只需要设置一次
            WriteSheet writeSheet = EasyExcel.writerSheet("帮我找房").build();
            List<Rent> list = new ArrayList<>();
            result.forEach(e -> {
                Rent rent = new Rent();
                BeanUtils.copyProperties(e,rent);
                list.add(rent);
            });
            excelWriter.write(list, writeSheet);
            // finish 会帮忙关闭流
            excelWriter.finish();
        }
    }

    /**
     * 保存对象
     * @param searchHouseDTO 实体对象
     * @return
     */
    @ApiOperation(value = "wx-保存帮我找房需求", notes = "wx-保存帮我找房需求")
    @RequestMapping(value="/wx/searchHouse",method= RequestMethod.POST)
    public ResponseBean searchHouseAdd(@RequestBody SearchHouseDTO searchHouseDTO,
                                       @RequestHeader("authorization") String token,
                                       HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        String openid = getOpenId(request);
        Integer orgId = getOrgId(request);
        List<TaPerson> persons = taPersonService.getPersonsByOpenId(openid);
        if (null == persons || persons.size() == 0) {
            return ResponseBean.error("当前账户信息异常, 清除缓存重试", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (StringUtils.isEmpty(searchHouseDTO.getMinPrice())) {
            return ResponseBean.error("请选择预算价格", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaPerson person = persons.get(0);
        try {
            SearchHouse searchHouse = new SearchHouse();
            BeanUtils.copyProperties(searchHouseDTO,searchHouse);
            searchHouse.setPersonId(person.getPersonId());
            searchHouse.setOrgId(orgId);
            searchHouse.setStatus("0");
            searchHouse.setCreatedTime(LocalDateTime.now());
            if (iSearchHouseService.save(searchHouse)){
                if (null != searchHouseDTO.getNeedRecommend() && searchHouseDTO.getNeedRecommend()) {
                    List<TaBuilding> buildingList = iSearchHouseService.getRecommendBuildings(getOrgId(request), searchHouseDTO);
                    if (CollectionUtils.isNotEmpty(buildingList)) {
                        for (TaBuilding build : buildingList) {
                            //查询封面当前图片
                            QueryWrapper<TaBuildingImg> queryListWrapper = new QueryWrapper<>();
                            queryListWrapper.eq("building_id", build.getBuildingId());
                            queryListWrapper.eq("img_type", CommConstant.BUILDING_IMG_LIST);
                            queryListWrapper.orderByAsc("create_date");

                            List<TaBuildingImg> buildingListImg = taBuildingImgService.list(queryListWrapper);
                            build.setBuildingListImg(buildingListImg);
                        }
                    }
                    SearchHouseVO searchHouseVO = new SearchHouseVO();
                    BeanUtils.copyProperties(searchHouse,searchHouseVO);
                    searchHouseVO.setTaBuildingList(buildingList);
                    responseBean.addSuccess(searchHouseVO);
                } else {
                    responseBean.addSuccess(null);
                }
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("searchHouseAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param searchHouse 实体对象
     * @return
     */
    @ApiOperation(value = "admin-修改帮我找房需求", notes = "admin-修改帮我找房需求")
    @RequestMapping(value="/admin/searchHouse/{id}",method= RequestMethod.PUT)
    public ResponseBean searchHouseUpdate(@PathVariable String id,
                                          @RequestBody SearchHouse searchHouse, @RequestHeader("authorization") String token, HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        try {
            searchHouse.setId(id);
            if (iSearchHouseService.updateById(searchHouse)){
                responseBean.addSuccess(searchHouse);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("searchHouseUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @ApiOperation(value = "admin-根据id查询帮我找房需求详情", notes = "admin-根据id查询帮我找房需求详情")
    @RequestMapping(value="/admin/searchHouse/{id}",method= RequestMethod.GET)
    public ResponseBean searchHouseGet(@PathVariable String id, @RequestHeader("authorization") String token, HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        try {
            SearchHouseVO searchHouseVO = iSearchHouseService.selectSearchHouseDetail(id);
            if (!StringUtils.isEmpty(searchHouseVO.getIntentArea())) {
                TdCity city = tdCityService.getById(searchHouseVO.getIntentArea());
                if(null != city){
                    searchHouseVO.setName(city.getName());
                }
            }
            responseBean.addSuccess(searchHouseVO);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("searchHouseDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }


    /**
     * 帮我找房回访接口
     * @param id  实体ID
     * @param searchHouseReplayDTO 实体对象
     * @return
     */
    @ApiOperation(value = "admin-帮我找房回访接口", notes = "admin-帮我找房回访接口")
    @RequestMapping(value="/admin/reply/{id}",method= RequestMethod.PUT)
    public ResponseBean searchHouseReply(@PathVariable String id,
                                         @RequestBody SearchHouseReplayDTO searchHouseReplayDTO, @RequestHeader("authorization") String token, HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        try {
            SearchHouse searchHouse = new SearchHouse();
            searchHouse.setId(id);
            searchHouse.setAuditRemark(searchHouseReplayDTO.getAuditRemark());
            searchHouse.setStatus(searchHouseReplayDTO.getStatus());
            searchHouse.setUpdatedTime(LocalDateTime.now());
            if (iSearchHouseService.updateById(searchHouse)){
                responseBean.addSuccess(searchHouse);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("searchHouseUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 获取帮我找房设置
     * @param request
     * @return
     */
    @GetMapping("/{plat}/searchHouse/setting")
    public ResponseBean getPhone(@PathVariable String plat,
                                 HttpServletRequest request) {
        Integer orgId = getOrgId(request);
        SysOrgParams paramByCode = iSysOrgParamsService.getParamByCode(CommConstant.SYSPARAM_FEEDBACK_PHONE, orgId);
        if (null == paramByCode) {
            return ResponseBean.error("客服热线未设置", ResponseBean.ERROR_UNAVAILABLE);
        }

        Map<String, Object> result = new HashMap<>();
        result.put("phone", paramByCode.getParamValue());

        return ResponseBean.success(result);
    }

    @PutMapping("/admin/searchHouse/setting/phone/{val}")
    public ResponseBean savePhone(@PathVariable String val, HttpServletRequest request) {
        if (StringUtils.isEmpty(val)) {
            return ResponseBean.error("请设置有效的电话热线", ResponseBean.ERROR_MISSING_PARAMS);
        }

        Integer orgId = getOrgId(request);
        SysOrgParams paramByCode = iSysOrgParamsService.getParamByCode(CommConstant.SYSPARAM_FEEDBACK_PHONE, orgId);
        if (null == paramByCode) {
            paramByCode = new SysOrgParams();
            paramByCode.setOrgId(orgId);
            paramByCode.setParamName("客服热线");
            paramByCode.setStatus(CommConstant.STATUS_NORMAL);
            paramByCode.setCreateDate(LocalDateTime.now());
            paramByCode.setParamValue(val);

            if (!iSysOrgParamsService.save(paramByCode)) {
               return ResponseBean.error("设置失败, 请重试!", ResponseBean.ERROR_MISSING_PARAMS);
            }
        } else {
            paramByCode.setParamValue(val);

            if (!iSysOrgParamsService.updateById(paramByCode)) {
                return ResponseBean.error("设置失败, 请重试!", ResponseBean.ERROR_MISSING_PARAMS);
            }
        }

        return ResponseBean.success("设置成功");
    }
}
